'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getChangedDeselection = exports.getChangedSelection = exports.getSelection = exports.rangeTo = exports.removeAll = exports.remove = exports.toggle = exports.replace = exports.setSelection = exports.setItems = exports.init = undefined;

var _fp = require('lodash/fp');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function init() {
  return {
    items: emptyIterable(),
    selected: [],
    changed: {
      selected: [],
      deselected: []
    },
    anchor: null
  };
}

var setItems = (0, _fp.curry)(function (items, state) {
  return {
    items: items,
    selected: state.selected,
    changed: {
      selected: [],
      deselected: []
    },
    anchor: state.anchor
  };
});

var setSelection = (0, _fp.curry)(function (selection, state) {
  return {
    items: state.items,
    selected: selection,
    changed: {
      selected: (0, _fp.without)(state.selected, selection),
      deselected: (0, _fp.without)(selection, state.selected)
    },
    anchor: state.anchor
  };
});

function getSelection(state) {
  return state.selected;
}

function getChangedSelection(state) {
  return state.changed.selected;
}

function getChangedDeselection(state) {
  return state.changed.deselected;
}

function getAnchor(state) {
  if (state.anchor !== null && state.anchor !== undefined) {
    return state.anchor;
  }

  if (state.selected.length > 0) {
    return getBottommostSelectedItem(state);
  }

  return state.items[Symbol.iterator]().next().value;
}

function getBottommostSelectedItem(state) {
  var previousItem = null;

  var isSelected = function isSelected(item) {
    return state.selected.indexOf(item) !== -1;
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = state.items[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var item = _step.value;

      if (isSelected(item)) {
        previousItem = item;
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return previousItem;
}

function emptyIterable() {
  var array = [];
  return _defineProperty({}, Symbol.iterator, array[Symbol.iterator].bind(array));
}

var replace = (0, _fp.curry)(function (selectedItem, state) {
  return {
    items: state.items,
    selected: [selectedItem],
    changed: {
      selected: (0, _fp.without)(state.selected, [selectedItem]),
      deselected: (0, _fp.without)([selectedItem], state.selected)
    },
    anchor: selectedItem
  };
});

var toggle = (0, _fp.curry)(function (toggledItem, state) {
  var itemIsAdded = !(0, _fp.includes)(toggledItem, state.selected);

  if (itemIsAdded) {
    return {
      items: state.items,
      selected: state.selected.concat([toggledItem]),
      changed: {
        selected: [toggledItem],
        deselected: []
      },
      anchor: toggledItem
    };
  }

  var anchorIsRemoved = toggledItem === state.anchor;
  var newAnchor = anchorIsRemoved ? null : state.anchor;

  return {
    items: state.items,
    selected: (0, _fp.without)([toggledItem], state.selected),
    changed: {
      selected: [],
      deselected: [toggledItem]
    },
    anchor: newAnchor
  };
});

var remove = (0, _fp.curry)(function (removedItem, state) {
  return {
    items: state.items,
    selected: (0, _fp.without)(removedItem, state.selected),
    changed: {
      selected: [],
      deselected: (0, _fp.intersection)(removedItem, state.selected)
    },
    anchor: null
  };
});

var removeAll = (0, _fp.curry)(function (state) {
  return {
    items: state.items,
    selected: [],
    changed: {
      selected: [],
      deselected: state.selected
    },
    anchor: null
  };
});

var rangeTo = (0, _fp.curry)(function (toItem, state) {
  var anchor = getAnchor(state, state.items);
  var connected = connectedWith(anchor, state.selected, state.items);
  var range = between(anchor, toItem, state.items);

  var selected = (0, _fp.flow)((0, _fp.without)(connected), (0, _fp.union)(range))(state.selected);

  return {
    items: state.items,
    selected: selected,
    changed: {
      selected: (0, _fp.without)(state.selected, selected),
      deselected: (0, _fp.without)(selected, state.selected)
    },
    anchor: state.anchor
  };
});

function between(start, end, iterable) {
  if (start === end) {
    return [start];
  }

  var result = [];
  var inRange = false;
  var foundStartEnd = 0;

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = iterable[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var item = _step2.value;

      if (inRange) {
        result.push(item);

        if (item === start || item === end) {
          foundStartEnd++;
          inRange = false;
        }
      } else {
        if (item === start || item === end) {
          foundStartEnd++;
          inRange = true;
          result.push(item);
        }
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  if (foundStartEnd === 2) {
    return result;
  }

  return [];
}

function connectedWith(targetItem, selected, iterable) {
  var range = [];
  var isRangeWithTargetItem = false;

  var isSelected = function isSelected(item) {
    return selected.indexOf(item) !== -1;
  };

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = iterable[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var item = _step3.value;

      if (isSelected(item)) {
        range.push(item);

        if (item === targetItem) {
          isRangeWithTargetItem = true;
        }
      } else {
        if (isRangeWithTargetItem) {
          break;
        } else {
          range = [];
        }
      }
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  return range;
}

exports.init = init;
exports.setItems = setItems;
exports.setSelection = setSelection;
exports.replace = replace;
exports.toggle = toggle;
exports.remove = remove;
exports.removeAll = removeAll;
exports.rangeTo = rangeTo;
exports.getSelection = getSelection;
exports.getChangedSelection = getChangedSelection;
exports.getChangedDeselection = getChangedDeselection;